/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia
  
  The DXC Framework is free software: you can redistribute it and/or modify it 
  under the terms of the GNU Lesser General Public License as published by the 
  Free Software Foundation, either version 3 of the License, or (at your 
  option) any later version. 

  This program is distributed in the hope that it will be useful, but WITHOUT 
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
  details. 

  You should have received a copy of the GNU Lesser General Public License 
  along with this program. If not, see <http://www.gnu.org/licenses/> 
 */
package org.dxc.standaloneSDS;

import org.dxc.api.DxcCallback;
import org.dxc.api.DxcDataGenerator;
import org.dxc.api.connection.ConnectorFactory;
import org.dxc.api.connection.DxcConnector;
import org.dxc.api.datatypes.*;
import org.dxc.api.parser.DxcParser;

import java.io.FileReader;
import java.io.PrintStream;
import java.util.LinkedList;
import java.util.List;

public class StandaloneSDS {
    private static void printUsage() {
        System.out.print("Usage: ");
        System.out.println(StandaloneSDS.class.getCanonicalName());
        System.out.println("                         "
                + "[-h ] [-a ] [-t waitTime] [-o output_file] input_file");
        System.out.println();
        System.out.println("       [-h]:             "
                + "display this message.");
        System.out.println("       [-a]:             "
                + "run in abbreviated mode, outputting data as");
        System.out.println("                         "
                + "quickly as possible (without sleep time ");
        System.out.println("                         " + "between messages).");
        System.out.println("       [-v system_name]: "
                + "validates data against specified system");
        System.out.println("                         "
                + "catalog, and loads probing/cost information.");
        System.out.println("       [-t wait_time]:   "
                + "specifies how long (seconds) StandaloneSDS ");
        System.out.println("                         "
                + "should wait after scenario end before exiting.");
        System.out.println("       [-o output_file]: "
                + "optional file to store results (otherwise");
        System.out.println("                         use stdout).");
        System.out.println("       input_file:       "
                + "scenario to run (must be in proper format).");
        System.out.println();
    }

    public static void main(String[] args) {
        try {
            long waitTime = 0;
            boolean abbrev = false;
            SystemCatalog sc = null;
            String outfile = null;
            if (args.length < 1) {
                printUsage();
            } else {
                int pos = 0;

                while (pos < args.length && args[pos].startsWith("-")) {
                    if (args[pos].equals("-h")) {
                        printUsage();
                        pos++;
                    } else if (args[pos].equals("-t")) {
                        try {
                            waitTime = Long.parseLong(args[++pos]);
                            pos++;
                        } catch (Exception ex) {
                            throw new Exception("-t: couldn't parse wait time.");
                        }
                    } else if (args[pos].equals("-a")) {
                        abbrev = true;
                        pos++;
                    } else if (args[pos].equals("-o")) {
                        outfile = args[++pos];
                        pos++;
                    } else if (args[pos].equals("-v")) {
                        sc = SystemCatalogXML.getInstance(args[++pos]);
                        pos++;
                    } else {
                        throw new Exception(args[pos] + ": unknown option.");
                    }
                }

                System.out.print("\n");

                if (pos >= args.length) {
                    throw new Exception("No input file specified.");
                }

                FileReader input = new FileReader(args[pos++]);

                PrintStream output;
                if (outfile != null) {
                    System.out.println("Writing output to: " + outfile);
                    output = new PrintStream(outfile);
                } else {
                    System.out.println("Writing output to stdout.");
                    output = System.out;
                }

                List<DxcCallback> cbl = new LinkedList<DxcCallback>();
                MultiCallback mcb = new MultiCallback(cbl);
                DxcConnector c = ConnectorFactory
                        .getStandaloneSDSConnector(mcb);

                WriteToFileCallback wcb = new WriteToFileCallback(output);
                cbl.add(wcb);
                cbl.add(new ProbeCallback(c));

                DataValidator dv = (sc == null) ? null : new DataValidator(sc);

                ForwardMessageCallback fmc = new ForwardMessageCallback(c, dv,
                        abbrev);

                synchronized (wcb) {
                    System.out.println("Awaiting DA...");
                    wcb.wait();
                }

                System.out.println("Starting.");
                c
                        .sendMessage(new ScenarioStatusData(
                                ScenarioStatusData.STARTED));

                DxcDataGenerator dgen = new DxcParser(input);
                dgen.begin(fmc);

                c.sendMessage(new ScenarioStatusData(ScenarioStatusData.ENDED));
                Thread.sleep(waitTime * 1000);
            }
        } catch (Exception ex) {
            System.out.println(ex.getMessage());
        }

        System.exit(0);
    }

}
