/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia

  The DXC Framework is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation, either version 3 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>
*/
#ifdef _WIN32
#include <vector>
#include <string.h>
#include <windows.h>
#include <psapi.h>

#include <DataTypes/DxcException.hpp>
#include <DataTypes/DxcData.hpp>
#include <WinExeLoader.hpp>

using namespace std;
using namespace Dxc;

WinExeLoader::WinExeLoader(const std::string& exeName, const std::string &path, const std::string& args)
    : exe_(exeName), path_(path), args_(args)
{
}

void WinExeLoader::start()
{
    STARTUPINFO si;
    ZeroMemory(&si, sizeof(si));
    si.cb = sizeof(si);
    ZeroMemory(&procinfo_, sizeof(procinfo_));

    if (exe_.find(".exe", exe_.length()-5) == string::npos)
    {
        exe_ += ".exe";
    }

    LPSTR name, path, args;
    string nameStr = path_ + DIR_DELIM + exe_;
    size_t nameLen = nameStr.length() + 1;
    size_t argsLen = args_.length() + nameLen + 3, pathLen = path_.length() + 1;
    name = new char[nameLen];
    args = new char[argsLen];
    path = new char[pathLen];

#pragma warning (disable:4996)
    strncpy(name, nameStr.c_str(), nameLen);
    strncpy(args, ("\"" + nameStr + "\"" ).c_str(), nameLen + 2);
    strncat(args, " ", 1);
    strncat(args, args_.c_str(), args_.length());
    strncpy(path, path_.c_str(), pathLen);
#pragma warning (disable:4996)

    int ret = CreateProcess(name,
                            args,        // command line
                            NULL,        // process security attributes
                            NULL,        // primary thread security attributes
                            false,       // handles are inherited
                            0,           // creation flags
                            NULL,        // use parent's environment
                            path,        // use parent's current directory
                            &si,         // STARTUPINFO pointer
                            &procinfo_); // receives PROCESS_INFORMATION

    delete name, path, args;

    if (ret == 0)
    {
        string err = "Unable to execute " + exe_ + ": " + getError();

        throw DxcException(err);
    }
}

bool WinExeLoader::isRunning()
{
    // check if process is still alive
    DWORD dwExitStatus = 0;

    GetExitCodeProcess(procinfo_.hProcess, &dwExitStatus);
    if (dwExitStatus == STILL_ACTIVE)
    {
        return true;
    }
    else
    {
        return false;
    }
}

bool WinExeLoader::terminate()
{
    DWORD dwExitStatus = 0;

    int ret = TerminateProcess(procinfo_.hProcess, 0);
    if (ret == 0)
    {
        string err = "Unable to terminate " + exe_ + ": " + getError();

        throw DxcException(err);
    }
    return false;
}

bool WinExeLoader::timedWait(long time)
{
    Timestamp t = DxcData::currentTime();
    while (isRunning())
    {
        Timestamp t1 = DxcData::currentTime();
        long dt = static_cast<long>(t1 - t);
        t = t1;

        long st = 1000 - dt;
        msleep((st > 0) ? st : 0);
        t1 = DxcData::currentTime();
        dt = static_cast<long>(t1 - t);
        time -= dt;
        t = DxcData::currentTime();
        if (time < 0)
        {
            return false;
        }
    }
    return true;
}

string WinExeLoader::getError()
{
    LPVOID lpMsgBuf;

    DWORD dw = GetLastError();
    FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |
                  FORMAT_MESSAGE_FROM_SYSTEM |
                  FORMAT_MESSAGE_IGNORE_INSERTS,
                  NULL,
                  dw,
                  MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
                  (LPTSTR)&lpMsgBuf,
                  0,
                  NULL);

    return (const char *)lpMsgBuf;
}

WinExeLoader::~WinExeLoader()
{
    if (isRunning())
    {
        terminate();
    }

    CloseHandle(procinfo_.hProcess);
}

unsigned long WinExeLoader::getCpuTime()
{
    FILETIME creationTime;
    FILETIME exitTime;
    FILETIME kernelTime;
    FILETIME userTime;

    if (!GetProcessTimes(procinfo_.hProcess,
                         &creationTime,
                         &exitTime,
                         &kernelTime,
                         &userTime))
    {
        string err = "Unable to get profile data: " + getError();

        throw DxcException(err);
    }
    long long uhdt = static_cast<long long>(userTime.dwHighDateTime);
    long long khdt = static_cast<long long>(kernelTime.dwHighDateTime);
    long long lKernelTime = (kernelTime.dwLowDateTime + (khdt << 32LL)) / 10000; // time in ms
    long long lUserTime = (userTime.dwLowDateTime + (uhdt << 32LL)) / 10000; // time in ms

    unsigned long time = static_cast<unsigned long>(lKernelTime + lUserTime);

    if ( ((lKernelTime + lUserTime) ^ time) != 0)
        throw DxcException("Data loss in cpu profiling.");

    return time;
}

unsigned long WinExeLoader::getPeakMemoryUsage()
{
    PROCESS_MEMORY_COUNTERS counters;

    if (!GetProcessMemoryInfo(procinfo_.hProcess, &counters, sizeof(PROCESS_MEMORY_COUNTERS)))
    {
        string err = "Unable to get profile data: " + getError();

        throw DxcException(err);
    }

    return counters.PeakWorkingSetSize;
}

#endif
